package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.PluginManagementService.ConnectorConfigWrapper;
import at.ac.uibk.gitsearch.service.PluginManagementService.ConnectorConfigWrapper.ActionWrapper;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import java.io.IOException;
import java.io.InputStream;
import org.codeability.sharing.plugins.api.SharingPluginConfig;
import org.codeability.sharing.plugins.api.SharingPluginConfig.Action;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.UserProvidedMetadataDTO;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.FileCopyUtils;

@SpringBootTest(classes = GitsearchApp.class)
public class PluginManagerActionIT {

    private final Logger log = LoggerFactory.getLogger(PluginManagerActionIT.class);
    private ConnectorConfigWrapper config;

    @BeforeEach
    public void initConfig() {
        config =
            new ConnectorConfigWrapper(
                new SharingPluginConfig(
                    "TestPlugin",
                    new Action[] {
                        new Action("someId", "someImportURL", "someActionName", "metadata.learningResourceType=='programming exercise'"),
                    }
                ),
                "http://localhost:8080/xxx"
            );
    }

    @Test
    public void testExpressionFiltersOnMetaData() throws JsonParseException, JsonMappingException, IOException {
        ActionWrapper actionWrapper = config.getActions().get("someId");

        Assert.assertTrue("metaData 1 should be applicable", actionWrapper.isApplicable(readTestResults("metaData/1/metaData.yaml")));
        Assert.assertTrue("metaData 272 should be applicable", actionWrapper.isApplicable(readTestResults("metaData/272/metaData.yaml")));
        Assert.assertFalse("metaData 2 should not be applicable", actionWrapper.isApplicable(readTestResults("metaData/2/metaData.yaml")));
    }

    /**
     * temporary static test data.
     *
     * @param infoString just a string to add to testdata description.
     * @return
     * @throws IOException
     * @throws JsonMappingException
     * @throws JsonParseException
     */
    private SearchResultDTO readTestResults(String testFile) throws JsonParseException, JsonMappingException, IOException {
        ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
        mapper.findAndRegisterModules();

        log.debug("reading test data from {} ", testFile);

        final String resourceName = "./testData/" + testFile;
        final InputStream resourceStream = this.getClass().getResourceAsStream(resourceName);
        Assert.assertNotNull("cannot find resource at " + resourceName, resourceStream);

        byte[] fileContent = FileCopyUtils.copyToByteArray(resourceStream);
        UserProvidedMetadataDTO metaData = mapper.readValue(fileContent, UserProvidedMetadataDTO.class);
        SearchResultDTO result = new SearchResultDTO();
        result.setMetadata(metaData);
        return result;
    }
}
